@@ -10,6 +10,7 @@
 @var RDF: to be used to set the return RDF Graph
 @var CSV: to be used to set the return format to CSV
 @var TSV: to be used to set the return format to TSV
+@var JSONLD: to be used to set the return format to JSON-LD
 
 @var POST: to be used to set HTTP POST
 @var GET: to be used to set HTTP GET. This is the default.
@@ -19,6 +20,20 @@
 @var ASK: to be used to set the query type to ASK. This is, usually, determined automatically.
 @var DESCRIBE: to be used to set the query type to DESCRIBE. This is, usually, determined automatically.
 
+@var INSERT: to be used to set the query type to INSERT.
+@var DELETE: to be used to set the query type to DELETE.
+@var CREATE: to be used to set the query type to CREATE.
+@var CLEAR: to be used to set the query type to CLEAR.
+@var DROP: to be used to set the query type to DROP.
+@var LOAD: to be used to set the query type to LOAD.
+@var COPY: to be used to set the query type to COPY.
+@var MOVE: to be used to set the query type to MOVE.
+@var ADD: to be used to set the query type to ADD.
+
+
+@var BASIC: BASIC HTTP Authentication method
+@var DIGEST: DIGEST HTTP Authentication method
+
 @see: U{SPARQL Specification<http://www.w3.org/TR/rdf-sparql-query/>}
 @authors: U{Ivan Herman<http://www.ivan-herman.net>}, U{Sergio Fernández<http://www.wikier.org>}, U{Carlos Tejo Alonso<http://www.dayures.net>}
 @organization: U{World Wide Web Consortium<http://www.w3.org>}, U{Salzburg Research<http://www.salzburgresearch.at>} and U{Foundation CTIC<http://www.fundacionctic.org/>}.
@@ -29,7 +44,6 @@
 import urllib.request, urllib.parse, urllib.error
 import urllib.request, urllib.error, urllib.parse
 from urllib.request import urlopen as urlopener  # don't change the name: tests override it
-import socket
 import base64
 import re
 import sys
@@ -37,7 +51,7 @@ import warnings
 
 import json
 from .KeyCaseInsensitiveDict import KeyCaseInsensitiveDict
-from .SPARQLExceptions import QueryBadFormed, EndPointNotFound, EndPointInternalError
+from .SPARQLExceptions import QueryBadFormed, EndPointNotFound, EndPointInternalError, Unauthorized
 from SPARQLWrapper import __agent__
 
 #  From <https://www.w3.org/TR/sparql11-protocol/#query-success>
@@ -48,7 +62,7 @@ from SPARQLWrapper import __agent__
 #  Possible parameter keys and values...
 #  Examples:
 #  - ClioPatria: the SWI-Prolog Semantic Web Server <http://cliopatria.swi-prolog.org/home>
-#    * Parameter key: "format"
+#    * Parameter key: "format" <http://cliopatria.swi-prolog.org/help/http>
 #    * Parameter value must have one of these values: "rdf+xml", "json", "csv", "application/sparql-results+xml" or "application/sparql-results+json".
 #
 #  - OpenLink Virtuoso  <http://virtuoso.openlinksw.com>
@@ -101,6 +115,7 @@ from SPARQLWrapper import __agent__
 #      - This table describes the MIME types and Accept Header/Output formats (MIME type) for different types of SPARQL queries. See <https://docs.marklogic.com/guide/semantics/REST#id_54258> and <https://docs.marklogic.com/guide/semantics/loading#id_70682>
 #        SELECT "application/sparql-results+xml", "application/sparql-results+json", "text/html", "text/csv"
 #        CONSTRUCT or DESCRIBE "application/n-triples", "application/rdf+json", "application/rdf+xml", "text/turtle", "text/n3", "application/n-quads", "application/trig"
+#        ASK queries return a boolean (true or false).
 #
 #  - AllegroGraph <https://franz.com/agraph/allegrograph/>
 #    * Uses only content negotiation (no URL parameters).
@@ -137,7 +152,7 @@ _allowedFormats = [JSON, XML, TURTLE, N3, RDF, RDFXML, CSV, TSV]
 
 # Possible HTTP methods
 POST = "POST"
-GET  = "GET"
+GET = "GET"
 _allowedRequests = [POST, GET]
 
 # Possible HTTP Authentication methods
@@ -165,7 +180,7 @@ _allowedQueryTypes = [SELECT, CONSTRUCT, ASK, DESCRIBE, INSERT, DELETE, CREATE,
 # Possible methods to perform requests
 URLENCODED = "urlencoded"
 POSTDIRECTLY = "postdirectly"
-_REQUEST_METHODS  = [URLENCODED, POSTDIRECTLY]
+_REQUEST_METHODS = [URLENCODED, POSTDIRECTLY]
 
 # Possible output format (mime types) that can be converted by the local script. Unfortunately,
 # it does not work by simply setting the return format, because there is still a certain level of confusion
@@ -177,7 +192,7 @@ _REQUEST_METHODS  = [URLENCODED, POSTDIRECTLY]
 # mime types have just been proposed and not yet widely used...
 _SPARQL_DEFAULT  = ["application/sparql-results+xml", "application/rdf+xml", "*/*"]
 _SPARQL_XML      = ["application/sparql-results+xml"]
-_SPARQL_JSON     = ["application/sparql-results+json", "text/javascript", "application/json"]
+_SPARQL_JSON     = ["application/sparql-results+json", "application/json", "text/javascript", "application/javascript"] # VIVO server returns "application/javascript"
 _RDF_XML         = ["application/rdf+xml"]
 _RDF_N3          = ["text/rdf+n3", "application/n-triples", "application/turtle", "application/n3", "text/n3", "text/turtle"]
 _RDF_JSONLD      = ["application/x-json+ld", "application/ld+json"]
@@ -186,7 +201,8 @@ _TSV             = ["text/tab-separated-values"]
 _XML             = ["application/xml"]
 _ALL             = ["*/*"]
 _RDF_POSSIBLE    = _RDF_XML + _RDF_N3 + _XML
-_SPARQL_PARAMS   = ["query"]
+
+_SPARQL_PARAMS = ["query"]
 
 try:
     import rdflib_jsonld
@@ -217,7 +233,24 @@ class SPARQLWrapper(object):
 
     @cvar pattern: regular expression used to determine whether a query is of type L{CONSTRUCT}, L{SELECT}, L{ASK}, or L{DESCRIBE}.
     @type pattern: compiled regular expression (see the C{re} module of Python)
-    @ivar baseURI: the URI of the SPARQL service
+    @ivar endpoint: SPARQL endpoint's URI
+    @type endpoint: string
+    @ivar updateEndpoint: SPARQL endpoint's URI for update operations (if it's a different one). Default is C{None}
+    @type updateEndpoint: string
+    @ivar agent: The User-Agent for the HTTP request header.
+    @type agent: string
+    @ivar _defaultGraph: URI for the default graph. Default is C{None}, the value can be set either via an L{explicit call<addParameter>}("default-graph-uri", uri) or as part of the query string.
+    @type _defaultGraph: string
+    @ivar user: The username of the credentials for querying the current endpoint. Default is C{None}, the value can be set an L{explicit call<setCredentials>}.
+    @type user: string
+    @ivar passwd: The password of the credentials for querying the current endpoint. Default is C{None}, the value can be set an L{explicit call<setCredentials>}.
+    @type passwd: string
+    @ivar http_auth: HTTP Authentication type. The default value is L{BASIC}. Possible values are L{BASIC} or L{DIGEST}
+    @type http_auth: string
+    @ivar onlyConneg: Option for allowing (or not) only HTTP Content Negotiation (so dismiss the use of HTTP parameters).The default value is L{False}.
+    @type onlyConneg: boolean
+    @ivar customHttpHeaders: Custom HTTP Headers to be included in the request. Important: These headers override previous values (including C{Content-Type}, C{User-Agent}, C{Accept} and C{Authorization} if they are present). It is a dictionary where keys are the header field nada and values are the header values.
+    @type customHttpHeaders: dict
     """
     pattern = re.compile(r"""
         ((?P<base>(\s*BASE\s*<.*?>)\s*)|(?P<prefixes>(\s*PREFIX\s+.+:\s*<.*?>)\s*))*
@@ -232,7 +265,7 @@ class SPARQLWrapper(object):
         @type endpoint: string
         @param updateEndpoint: string of the SPARQL endpoint's URI for update operations (if it's a different one)
         @type updateEndpoint: string
-        @keyword returnFormat: Default: L{XML}.
+        @param returnFormat: Default: L{XML}.
         Can be set to JSON or Turtle/N3
 
         No local check is done, the parameter is simply
@@ -243,8 +276,10 @@ class SPARQLWrapper(object):
         L{JSON}, L{XML}, L{TURTLE}, L{N3}, L{RDFXML}, L{CSV}, L{TSV} (constants in this module). The value can also be set via explicit
         call, see below.
         @type returnFormat: string
-        @keyword defaultGraph: URI for the default graph. Default is None, the value can be set either via an L{explicit call<addDefaultGraph>} or as part of the query string.
+        @param defaultGraph: URI for the default graph. Default is None, the value can be set either via an L{explicit call<addDefaultGraph>} or as part of the query string.
         @type defaultGraph: string
+        @param agent: The User-Agent for the HTTP request header.
+        @type agent: string
         """
         self.endpoint = endpoint
         self.updateEndpoint = updateEndpoint if updateEndpoint else endpoint
@@ -254,6 +289,7 @@ class SPARQLWrapper(object):
         self.http_auth = BASIC
         self._defaultGraph = defaultGraph
         self.onlyConneg = False # Only Content Negotiation
+        self.customHttpHeaders = {}
 
         if returnFormat in _allowedFormats:
             self._defaultReturnFormat = returnFormat
@@ -263,7 +299,7 @@ class SPARQLWrapper(object):
         self.resetQuery()
 
     def resetQuery(self):
-        """Reset the query, ie, return format, query, default or named graph settings, etc,
+        """Reset the query, ie, return format, method, query, default or named graph settings, etc,
         are reset to their default values.
         """
         self.parameters = {}
@@ -275,13 +311,15 @@ class SPARQLWrapper(object):
         self.timeout = None
         self.requestMethod = URLENCODED
 
+
     def setReturnFormat(self, format):
         """Set the return format. If not an allowed value, the setting is ignored.
 
-        @param format: Possible values: are L{JSON}, L{XML}, L{TURTLE}, L{N3}, L{RDF}, L{RDFXML}, L{CSV}, L{TSV} (constants in this module). All other cases are ignored.
-        @type format: str
+        @param format: Possible values are L{JSON}, L{XML}, L{TURTLE}, L{N3}, L{RDF}, L{RDFXML}, L{CSV}, L{TSV}, L{JSONLD} (constants in this module). All other cases are ignored.
+        @type format: string
+        @raise ValueError: If L{JSONLD} is tried to set and the current instance does not support JSON-LD.
         """
-        if format in _allowedFormats :
+        if format in _allowedFormats:
             self.returnFormat = format
         elif format == JSONLD:
             raise ValueError("Current instance does not support JSON-LD; you might want to install the rdflib-json package.")
@@ -291,8 +329,10 @@ class SPARQLWrapper(object):
     def supportsReturnFormat(self, format):
         """Check if a return format is supported.
 
-        @param format: Possible values: are L{JSON}, L{XML}, L{TURTLE}, L{N3}, L{RDF}, L{RDFXML}, L{CSV}, L{TSV} (constants in this module). All other cases are ignored.
-        @type format: bool
+        @param format: Possible values are L{JSON}, L{XML}, L{TURTLE}, L{N3}, L{RDF}, L{RDFXML}, L{CSV}, L{TSV} (constants in this module). All other cases are ignored.
+        @type format: string
+        @return: Returns a boolean after checking if a return format is supported.
+        @rtype: bool
         """
         return (format in _allowedFormats)
 
@@ -306,6 +346,7 @@ class SPARQLWrapper(object):
 
     def setOnlyConneg(self, onlyConneg):
         """Set this option for allowing (or not) only HTTP Content Negotiation (so dismiss the use of HTTP parameters).
+        @since: 1.8.1
 
         @param onlyConneg: True if only HTTP Content Negotiation is allowed; False is HTTP parameters are allowed also.
         @type onlyConneg: bool
@@ -319,8 +360,8 @@ class SPARQLWrapper(object):
         Further details at U{http://www.w3.org/TR/sparql11-protocol/#query-operation}
         and U{http://www.w3.org/TR/sparql11-protocol/#update-operation}.
 
-        @param method: method
-        @type method: str
+        @param method: Possible values are C{SPARQLWrapper.URLENCODED} (URL-encoded) or C{SPARQLWrapper.POSTDIRECTLY} (POST directly). All other cases are ignored.
+        @type method: string
         """
         if method in _REQUEST_METHODS:
             self.requestMethod = method
@@ -350,6 +391,7 @@ class SPARQLWrapper(object):
             Some SPARQL endpoints require extra key value pairs.
             E.g., in virtuoso, one would add C{should-sponge=soft} to the query forcing
             virtuoso to retrieve graphs that are not stored in its local database.
+            Alias of L{SPARQLWrapper.addParameter} method.
             @param key: key of the query part
             @type key: string
             @param value: value of the query part
@@ -360,11 +402,12 @@ class SPARQLWrapper(object):
 
     def addCustomParameter(self, name, value):
         """
-            Method is kept for backwards compatibility. Historically, it "replaces" parameters instead of adding
+            Method is kept for backwards compatibility. Historically, it "replaces" parameters instead of adding.
             @param name: name
             @type name: string
             @param value: value
             @type value: string
+            @return: Returns a boolean indicating if the adding has been accomplished.
             @rtype: bool
             @deprecated: use addParameter(name, value) instead of this method
         """
@@ -376,10 +419,13 @@ class SPARQLWrapper(object):
             Some SPARQL endpoints allow extra key value pairs.
             E.g., in virtuoso, one would add C{should-sponge=soft} to the query forcing
             virtuoso to retrieve graphs that are not stored in its local database.
+            If the param C{query} is tried to be set, this intent is dismissed.
+            Returns a boolean indicating if the set has been accomplished.
             @param name: name
             @type name: string
             @param value: value
             @type value: string
+            @return: Returns a boolean indicating if the adding has been accomplished.
             @rtype: bool
         """
         if name in _SPARQL_PARAMS:
@@ -390,11 +436,45 @@ class SPARQLWrapper(object):
             self.parameters[name].append(value)
             return True
 
+    def addCustomHttpHeader(self, httpHeaderName, httpHeaderValue):
+        """
+            Add a custom HTTP header (this method can override all HTTP headers).
+            IMPORTANT: Take into acount that each previous value for the header field names
+            C{Content-Type}, C{User-Agent}, C{Accept} and C{Authorization} would be overriden
+            if the header field name is present as value of the parameter C{httpHeaderName}.
+            @since: 1.8.2
+
+            @param httpHeaderName: The header field name.
+            @type httpHeaderName: string
+            @param httpHeaderValue: The header field value.
+            @type httpHeaderValue: string
+        """
+        self.customHttpHeaders[httpHeaderName] = httpHeaderValue
+
+    def clearCustomHttpHeader(self, httpHeaderName):
+        """
+            Clear the values of a custom Http Header previously setted.
+            Returns a boolean indicating if the clearing has been accomplished.
+            @since: 1.8.2
+
+            @param httpHeaderName: name
+            @type httpHeaderName: string
+            @return: Returns a boolean indicating if the clearing has been accomplished.
+            @rtype: bool
+        """
+        try:
+            del self.customHttpHeaders[httpHeaderName]
+            return True
+        except KeyError:
+            return False
+
     def clearParameter(self, name):
         """
-            Clear the values ofd a concrete parameter.
+            Clear the values of a concrete parameter.
+            Returns a boolean indicating if the clearing has been accomplished.
             @param name: name
             @type name: string
+            @return: Returns a boolean indicating if the clearing has been accomplished.
             @rtype: bool
         """
         if name in _SPARQL_PARAMS:
@@ -408,7 +488,7 @@ class SPARQLWrapper(object):
 
     def setCredentials(self, user, passwd):
         """
-            Set the credentials for querying the current endpoint
+            Set the credentials for querying the current endpoint.
             @param user: username
             @type user: string
             @param passwd: password
@@ -419,9 +499,11 @@ class SPARQLWrapper(object):
 
     def setHTTPAuth(self, auth):
         """
-           Set the HTTP Authentication type (Basic or Digest)
-           @param auth: auth type
-           @type auth: string
+            Set the HTTP Authentication type. Possible values are L{BASIC} or L{DIGEST}.
+            @param auth: auth type
+            @type auth: string
+            @raise TypeError: If the C{auth} parameter is not an string.
+            @raise ValueError: If the C{auth} parameter has not one of the valid values: L{BASIC} or L{DIGEST}.
         """
         if not isinstance(auth, str):
             raise TypeError('setHTTPAuth takes a string')
@@ -438,7 +520,7 @@ class SPARQLWrapper(object):
             ASK, etc). Syntax and validity checking is done by the SPARQL service itself.
             @param query: query text
             @type query: string
-            @bug: #2320024
+            @raise TypeError: If the C{query} parameter is not an unicode-string or utf-8 encoded byte-string.
         """
         if sys.version < '3':  # have to write it like this, for 2to3 compatibility
             if isinstance(query, str):
@@ -456,11 +538,11 @@ class SPARQLWrapper(object):
                 raise TypeError('setQuery takes either unicode-strings or utf-8 encoded byte-strings')
 
         self.queryString = query
-        self.queryType   = self._parseQueryType(query)
+        self.queryType = self._parseQueryType(query)
 
-    def _parseQueryType(self,query):
+    def _parseQueryType(self, query):
         """
-            Parse the SPARQL query and return its type (ie, L{SELECT}, L{ASK}, etc).
+            Internal method for parsing the SPARQL query and return its type (ie, L{SELECT}, L{ASK}, etc).
 
             Note that the method returns L{SELECT} if nothing is specified. This is just to get all other
             methods running; in fact, this means that the query is erroneous, because the query must be,
@@ -469,28 +551,31 @@ class SPARQLWrapper(object):
 
             @param query: query text
             @type query: string
+            @return: the type of SPARQL query (aka SPARQL query form)
             @rtype: string
         """
         try:
-            query = query if type(query)==str else query.encode('ascii', 'ignore')
+            query = query if (isinstance(query, str)) else query.encode('ascii', 'ignore')
             query = self._cleanComments(query)
             r_queryType = self.pattern.search(query).group("queryType").upper()
         except AttributeError:
             warnings.warn("not detected query type for query '%s'" % query.replace("\n", " "), RuntimeWarning)
             r_queryType = None
 
-        if r_queryType in _allowedQueryTypes :
+        if r_queryType in _allowedQueryTypes:
             return r_queryType
-        else :
+        else:
             #raise Exception("Illegal SPARQL Query; must be one of SELECT, ASK, DESCRIBE, or CONSTRUCT")
             warnings.warn("unknown query type '%s'" % r_queryType, RuntimeWarning)
             return SELECT
 
-    def setMethod(self,method):
+    def setMethod(self, method):
         """Set the invocation method. By default, this is L{GET}, but can be set to L{POST}.
         @param method: should be either L{GET} or L{POST}. Other cases are ignored.
+        @type method: string
         """
-        if method in _allowedRequests : self.method = method
+        if method in _allowedRequests:
+            self.method = method
 
     def setUseKeepAlive(self):
         """Make urllib2 use keep-alive.
@@ -510,25 +595,36 @@ class SPARQLWrapper(object):
             warnings.warn("keepalive support not available, so the execution of this method has no effect")
 
     def isSparqlUpdateRequest(self):
-        """ Returns TRUE if SPARQLWrapper is configured for executing SPARQL Update request
-        @return: bool
+        """ Returns C{TRUE} if SPARQLWrapper is configured for executing SPARQL Update request.
+        @return: Returns C{TRUE} if SPARQLWrapper is configured for executing SPARQL Update request
+        @rtype: bool
         """
         return self.queryType in [INSERT, DELETE, CREATE, CLEAR, DROP, LOAD, COPY, MOVE, ADD]
 
     def isSparqlQueryRequest(self):
-        """ Returns TRUE if SPARQLWrapper is configured for executing SPARQL Query request
-        @return: bool
+        """ Returns C{TRUE} if SPARQLWrapper is configured for executing SPARQL Query request.
+        @return: Returns C{TRUE} if SPARQLWrapper is configured for executing SPARQL Query request.
+        @rtype: bool
         """
         return not self.isSparqlUpdateRequest()
 
     def _cleanComments(self, query):
-        # remove all occurrence singleline comments (issues #32 and #77)
-        return re.sub(self.comments_pattern, "\n\n" , query)
+        """ Internal method for returning the query after all occurrence of singleline comments are removed (issues #32 and #77).
+        @param query: The query
+        @type query: string
+        @return: the query after all occurrence of singleline comments are removed.
+        @rtype: string
+        """
+        return re.sub(self.comments_pattern, "\n\n", query)
 
     def _getRequestEncodedParameters(self, query=None):
+        """ Internal method for getting the request encoded parameters.
+        @param query: The query
+        @type query: string
+        """
         query_parameters = self.parameters.copy()
 
-        if query and type(query) == tuple and len(query) == 2:
+        if query and (isinstance(query, tuple)) and len(query) == 2:
             #tuple ("query"/"update", queryString)
             query_parameters[query[0]] = [query[1]]
 
@@ -545,8 +641,8 @@ class SPARQLWrapper(object):
                 if self.returnFormat in [TSV, JSONLD]:
                     acceptHeader = self._getAcceptHeader() # to obtain the mime-type "text/tab-separated-values"
                     if "*/*" in acceptHeader:
-                        acceptHeader="" # clear the value in case of "*/*"
-                    query_parameters[f]+= [acceptHeader]
+                        acceptHeader = "" # clear the value in case of "*/*"
+                    query_parameters[f] += [acceptHeader]
 
         pairs = (
             "%s=%s" % (
@@ -559,6 +655,9 @@ class SPARQLWrapper(object):
         return '&'.join(pairs)
 
     def _getAcceptHeader(self):
+        """ Internal method for getting the HTTP Accept Header.
+        @see: U{Hypertext Transfer Protocol -- HTTP/1.1 - Header Field Definitions<https://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.1>}
+        """
         if self.queryType in [SELECT, ASK]:
             if self.returnFormat == XML:
                 acceptHeader = ",".join(_SPARQL_XML)
@@ -588,6 +687,7 @@ class SPARQLWrapper(object):
     def _createRequest(self):
         """Internal method to create request according a HTTP method. Returns a
         C{urllib2.Request} object of the urllib2 Python library
+        @raise NotImplementedError: If the C{HTTP authentification} method is not one of the valid values: L{BASIC} or L{DIGEST}.
         @return: request a C{urllib2.Request} object of the urllib2 Python library
         """
         request = None
@@ -641,26 +741,37 @@ class SPARQLWrapper(object):
                 raise NotImplementedError("Expecting one of: {0}, but received: {1}".format(valid_types,
                                                                                             self.http_auth))
 
+        # The header field name is capitalized in the request.add_header method.
+        for customHttpHeader in self.customHttpHeaders:
+            request.add_header(customHttpHeader, self.customHttpHeaders[customHttpHeader])
+
         return request
 
     def _query(self):
         """Internal method to execute the query. Returns the output of the
         C{urllib2.urlopen} method of the standard Python library
 
-        @return: tuples with the raw request plus the expected format
+        @return: tuples with the raw request plus the expected format.
+        @raise QueryBadFormed: If the C{HTTP return code} is C{400}.
+        @raise Unauthorized: If the C{HTTP return code} is C{401}.
+        @raise EndPointNotFound: If the C{HTTP return code} is C{404}.
+        @raise EndPointInternalError: If the C{HTTP return code} is C{500}.
         """
-        if self.timeout:
-            socket.setdefaulttimeout(self.timeout)
         request = self._createRequest()
 
         try:
-            response = urlopener(request)
+            if self.timeout:
+                response = urlopener(request, timeout=self.timeout)
+            else:
+                response = urlopener(request)
             return response, self.returnFormat
         except urllib.error.HTTPError as e:
             if e.code == 400:
                 raise QueryBadFormed(e.read())
             elif e.code == 404:
                 raise EndPointNotFound(e.read())
+            elif e.code == 401:
+                raise Unauthorized(e.read())
             elif e.code == 500:
                 raise EndPointInternalError(e.read())
             else:
@@ -715,12 +826,15 @@ class QueryResult(object):
     would work, too.
 
     @ivar response: the direct HTTP response; a file-like object, as return by the C{urllib2.urlopen} library call.
+    @ivar requestedFormat: The requested format. The possible values are: L{JSON}, L{XML}, L{RDFXML}, L{TURTLE}, L{N3}, L{RDF}, L{CSV}, L{TSV}, L{JSONLD}.
+    @type requestedFormat: string
+
     """
-    def __init__(self,result):
+    def __init__(self, result):
         """
         @param result: HTTP response stemming from a L{SPARQLWrapper.query} call, or a tuple with the expected format: (response,format)
         """
-        if (type(result) == tuple):
+        if isinstance(result, tuple):
             self.response = result[0]
             self.requestedFormat = result[1]
         else:
@@ -728,16 +842,16 @@ class QueryResult(object):
         """Direct response, see class comments for details"""
 
     def geturl(self):
-        """Return the URI of the original call.
-        @return: URI
+        """Return the URL of the original call.
+        @return: URL of the original call
         @rtype: string
         """
         return self.response.geturl()
 
     def info(self):
         """Return the meta-information of the HTTP result.
-        @return: meta information
-        @rtype: dictionary
+        @return: meta information of the HTTP result
+        @rtype: dict
         """
         return KeyCaseInsensitiveDict(self.response.info())
 
@@ -756,7 +870,7 @@ class QueryResult(object):
         Convert a JSON result into a Python dict. This method can be overwritten in a subclass
         for a different conversion method.
         @return: converted result
-        @rtype: Python dictionary
+        @rtype: dict
         """
         return json.loads(self.response.read().decode("utf-8"))
 
@@ -775,7 +889,7 @@ class QueryResult(object):
         Convert a RDF/XML result into an RDFLib triple store. This method can be overwritten
         in a subclass for a different conversion method.
         @return: converted result
-        @rtype: RDFLib Graph
+        @rtype: RDFLib C{Graph}
         """
         try:
             from rdflib.graph import ConjunctiveGraph
@@ -816,7 +930,7 @@ class QueryResult(object):
 
     def _convertJSONLD(self):
         """
-        Convert a RDF JSON-LDresult into an RDFLib triple store. This method can be overwritten
+        Convert a RDF JSON-LD result into an RDFLib triple store. This method can be overwritten
         in a subclass for a different conversion method.
         @return: converted result
         @rtype: RDFLib Graph
@@ -831,7 +945,7 @@ class QueryResult(object):
         Encode the return value depending on the return format:
             - in the case of XML, a DOM top element is returned;
             - in the case of JSON, a simplejson conversion will return a dictionary;
-            - in the case of RDF/XML, the value is converted via RDFLib into a Graph instance;
+            - in the case of RDF/XML, the value is converted via RDFLib into a C{Graph} instance;
             - in the case of RDF Turtle/N3, a string is returned;
             - in the case of CSV/TSV, a string is returned.
         In all other cases the input simply returned.
@@ -874,37 +988,37 @@ class QueryResult(object):
             elif _content_type_in_list(ct, _RDF_JSONLD):
                 _validate_format("JSON(-LD)", [JSONLD, JSON], ct, self.requestedFormat)
                 return self._convertJSONLD()
-
-        warnings.warn("unknown response content type '%s' returning raw response..." %(ct), RuntimeWarning)
+            else:
+                warnings.warn("unknown response content type '%s' returning raw response..." %(ct), RuntimeWarning)
         return self.response.read()
 
     def print_results(self, minWidth=None):
         results = self._convertJSON()
-        if minWidth :
+        if minWidth:
             width = self.__get_results_width(results, minWidth)
-        else :
+        else:
             width = self.__get_results_width(results)
         index = 0
-        for var in results["head"]["vars"] :
-            print(("?" + var).ljust(width[index]),"|", end=' ')
+        for var in results["head"]["vars"]:
+            print(("?" + var).ljust(width[index]), "|", end=' ')
             index += 1
         print()
         print("=" * (sum(width) + 3 * len(width)))
-        for result in results["results"]["bindings"] :
+        for result in results["results"]["bindings"]:
             index = 0
-            for var in results["head"]["vars"] :
+            for var in results["head"]["vars"]:
                 result = self.__get_prettyprint_string_sparql_var_result(result[var])
-                print(result.ljust(width[index]),"|", end=' ')
+                print(result.ljust(width[index]), "|", end=' ')
                 index += 1
             print()
 
     def __get_results_width(self, results, minWidth=2):
         width = []
-        for var in results["head"]["vars"] :
+        for var in results["head"]["vars"]:
             width.append(max(minWidth, len(var)+1))
-        for result in results["results"]["bindings"] :
+        for result in results["results"]["bindings"]:
             index = 0
-            for var in results["head"]["vars"] :
+            for var in results["head"]["vars"]:
                 result = self.__get_prettyprint_string_sparql_var_result(result[var])
                 width[index] = max(width[index], len(result))
                 index += 1
@@ -913,9 +1027,9 @@ class QueryResult(object):
     def __get_prettyprint_string_sparql_var_result(self, result):
         value = result["value"]
         lang = result.get("xml:lang", None)
-        datatype = result.get("datatype",None)
+        datatype = result.get("datatype", None)
         if lang is not None:
-            value+="@"+lang
+            value += "@"+lang
         if datatype is not None:
-            value+=" ["+datatype+"]"
+            value += " ["+datatype+"]"
         return value
